package com.hikcreate.drv_photo_pic.impl;


import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import com.hikcreate.dao.DriverPhotoMapper;
import com.hikcreate.drv_photo_pic.DrvPhoto;
import com.hikcreate.entity.DriverPhoto;
import com.hikcreate.entity.PicResult;
import com.hikcreate.service.fdfs.service.FileService;
import com.hikcreate.utils.HbaseUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.hadoop.hbase.client.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.zip.CRC32;


/**
 * @author lifuyi
 * @date ${Date}
 * @description
 */
@Service("drvPhotoImpl")
public class DrvPhotoImpl implements DrvPhoto {

    private static Logger logger = LoggerFactory.getLogger(VehicleImpl.class);

    @Value("${roundDay}")
    private int roundDay;
    @Autowired
    private FileService fileService;
    @Autowired
    private SqlHelp sqlHelp;
    @Autowired
    private HbaseUtils hbaseUtils;
    @Autowired
    private DriverPhotoMapper driverPhotoMapper;
    @Autowired
    private FastFileStorageClient storageClient;


    @Autowired
    private KafkaTemplate kafkaTemplate;
    @Value("${hbase.zookeeper.property.clientPort}")
    private String hbaseAddress;
    @Value("${hbase.zookeeper.quorum}")
    private String hbaseZkQuorum;
    @Value("${hbase.master}")
    private String hbaseMaster;
    @Value("${hbase.drv_photo.table}")
    private String driverPhotoTableStr;
    @Value("pic.topic")
    private String picTopic;

    private static CRC32 crc = new CRC32();


    /**
     * 每天增量同步
     */
    @Override
    public void getIncrementDrvPhoto() {
        org.apache.hadoop.hbase.client.Connection hbseCon = null;
        Table driverPhototable = null;
        try {
            hbseCon = hbaseUtils.getHbseCon(hbaseAddress, hbaseZkQuorum);
            driverPhototable = hbaseUtils.getHbaseTable(hbseCon, driverPhotoTableStr);
            List<DriverPhoto> driverPhotos = driverPhotoMapper.getIncrePhoto();
            for (DriverPhoto driverPhoto : driverPhotos) {
                saveToHbase(driverPhoto, driverPhototable);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            hbaseUtils.closeHbaseTable(driverPhototable);
            hbaseUtils.clodeHbaseCon(hbseCon);
        }
    }

    /**
     * 插入指定驾驶人头像
     *
     * @param sfzmhm
     */
    @Override
    public void insertDrvPhotoById(String sfzmhm) {
        org.apache.hadoop.hbase.client.Connection hbseCon = null;
        Table driverPhototable = null;
        try {
            hbseCon = hbaseUtils.getHbseCon(hbaseAddress, hbaseZkQuorum);
            driverPhototable = hbaseUtils.getHbaseTable(hbseCon, driverPhotoTableStr);
            List<DriverPhoto> driverPhotos = driverPhotoMapper.getIncrePhotoBySfzmhm(sfzmhm);
            saveToHbase(driverPhotos.get(0), driverPhototable);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            hbaseUtils.closeHbaseTable(driverPhototable);
            hbaseUtils.clodeHbaseCon(hbseCon);
        }
    }

    /**
     * 同步指定时间段的驾驶人头像
     *
     * @param start
     * @param end
     */
    @Override
    public void increDrvPic(String start, String end) {
        org.apache.hadoop.hbase.client.Connection hbseCon = null;
        Table driverPhototable = null;
        try {
            hbseCon = hbaseUtils.getHbseCon(hbaseAddress, hbaseZkQuorum);
            driverPhototable = hbaseUtils.getHbaseTable(hbseCon, driverPhotoTableStr);
            List<DriverPhoto> driverPhotos = driverPhotoMapper.getIncrePhotoStage(start, end);
            for (DriverPhoto driverPhoto : driverPhotos) {
                saveToHbase(driverPhoto, driverPhototable);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            hbaseUtils.closeHbaseTable(driverPhototable);
            hbaseUtils.clodeHbaseCon(hbseCon);
        }
    }

    /**
     * 同步发证机关为null的指定时间的数据
     * @param start
     * @param end
     */
    @Override
    public void increDrvPicFzjgNull(String start, String end) {
        org.apache.hadoop.hbase.client.Connection hbseCon = null;
        Table driverPhototable = null;
        try {
            hbseCon = hbaseUtils.getHbseCon(hbaseAddress, hbaseZkQuorum);
            driverPhototable = hbaseUtils.getHbaseTable(hbseCon, driverPhotoTableStr);
            List<DriverPhoto> driverPhotos = driverPhotoMapper.increDrvPicFzjgNull(start, end);
            for (DriverPhoto driverPhoto : driverPhotos) {
                saveToHbase(driverPhoto, driverPhototable);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            hbaseUtils.closeHbaseTable(driverPhototable);
            hbaseUtils.clodeHbaseCon(hbseCon);
        }
    }

    private synchronized void saveToHbase(DriverPhoto driverPhoto, Table driverPhototable) {
        try {
            //判断图片是否已经存入fastdfs
            String[] colValue = new String[2];
            String rowkey = driverPhoto.getSfzmhm();
            PicResult picResult = sqlHelp.getFlag(rowkey, driverPhototable, colValue, driverPhoto.getGxsj());
            if (picResult.getI() != 0) {
//            if (picResult.getI() == 1) {
                if (picResult.getI() == 2) {
                    try {
                        fileService.deleteFile(picResult.getUrl());
                    } catch (Exception e) {
                        logger.info(driverPhoto.getSfzmhm() + "---------------------" + picResult.getUrl());
                    } finally {
                        Delete delete = new Delete(rowkey.getBytes());
                        driverPhototable.delete(delete);
                    }
                }
                Put put = new Put(rowkey.getBytes());
                put.addColumn("info".getBytes(), "sfzmhm".getBytes(), driverPhoto.getSfzmhm().getBytes());
                //上传
                int count = 0;
                String picUrl = uploadFileStor(driverPhoto.getZp(), driverPhoto.getSfzmhm());
                while (picUrl == null) {
                    picUrl = uploadFileStor(driverPhoto.getZp(), driverPhoto.getSfzmhm());
                    count++;
                    if (count > 10) {
                        logger.info("10fail--------sfzmhm---------" + driverPhoto.getSfzmhm());
                        return;
                    }
                }
                //验证
                byte[] bytesDown = fileService.downloadFile("/" + picUrl);
                if (bytesDown.length != driverPhoto.getZp().length) {
                    logger.info("length not right driver:-----------" + driverPhoto.getSfzmhm() + "----------" + bytesDown.length);
                    picUrl = uploadFileStor(driverPhoto.getZp(), driverPhoto.getSfzmhm());
                }
                //构造put
                put.addColumn("info".getBytes(), "picLen".getBytes(), String.valueOf(driverPhoto.getZp().length / 1024).getBytes());
                put.addColumn("info".getBytes(), "sfzmhm".getBytes(), (driverPhoto.getSfzmhm() == null ? "null" : driverPhoto.getSfzmhm()).getBytes());
                put.addColumn("info".getBytes(), "picUrl".getBytes(), ("/" + picUrl).getBytes());
                put.addColumn("info".getBytes(), "gxsj".getBytes(), (driverPhoto.getGxsj() == null ? "null" : driverPhoto.getGxsj()).getBytes());
                put.addColumn("info".getBytes(), "xzqh".getBytes(), (driverPhoto.getXzqh() == null ? "null" : driverPhoto.getXzqh()).getBytes());
                put.addColumn("info".getBytes(), "flag".getBytes(), (driverPhoto.getFlag() == null ? "null" : driverPhoto.getFlag()).getBytes());
                put.addColumn("info".getBytes(), "xh".getBytes(), (driverPhoto.getXh() == null ? "null" : driverPhoto.getXh()).getBytes());
                put.addColumn("info".getBytes(), "rksj".getBytes(), (driverPhoto.getRksj() == null ? "null" : driverPhoto.getRksj()).getBytes());
                put.addColumn("info".getBytes(), "fzjg".getBytes(), (driverPhoto.getFzjg() == null ? "null" : driverPhoto.getFzjg()).getBytes());
                driverPhototable.put(put);
/*                JSONObject resultJson = new JSONObject();
                resultJson.put("type","drv");
                resultJson.put("row",driverPhoto.getSfzmhm());
                resultJson.put("picUrl","/" + picUrl);
                resultJson.put("info",driverPhoto.getSfzmhm());
                kafkaTemplate.send(picTopic,resultJson.toString());*/
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("error111");
        }
    }

    private String uploadFileStor(byte[] bytes, String sfzmhm) {
        try {
            StorePath jpeg = storageClient.uploadFile(new ByteArrayInputStream(bytes), bytes.length, "jpeg", null);
            return jpeg.getFullPath();
        } catch (Exception e) {
            logger.info(sfzmhm + "***************************");
            return null;
        }
    }


    public static String getPrefixString(long prefixLong) {
        if (prefixLong < 10L) {
            return "00" + prefixLong;
        } else if (prefixLong >= 10L && prefixLong < 100L) {
            return "0" + prefixLong;
        } else {
            return "" + prefixLong;
        }
    }




}
